require "defines"

local TIME_LIMIT = 60*60*60 * 24
local CHESTINV = defines.inventory.chest
local SUBMISSION_ENTITY="logistic-chest-active-provider"
local floor, format = math.floor, string.format

local BUILDEVENTS = {defines.events.on_built_entity,defines.events.on_robot_built_entity}

local forces, queue
function alias()
	forces = global.forces
	queue = global.queue
end

script.on_init(function()
	if not global.forces then global.forces = {} end
	if not global.queue then global.queue = {} end
	alias()
end)

script.on_load(function()
	alias()
end)

function broadcast(force, message)
	for _,p in pairs(force.players) do
		p.print(message)
	end
end

function addyield(list,entity)
	if entity.mineable_properties and entity.mineable_properties.products then
		for _,product in pairs(entity.mineable_properties.products) do
			if product.type=="item" then list[product.name]=true end
		end
	end
	if entity.loot then
		for _,product in pairs(entity.loot) do
			list[product.item]=true
		end
	end	
end

function givestartup(player, leader)
	player.insert{name="pistol", count=1}
	if leader then
		player.insert{name="iron-plate", count=8}
		player.insert{name="burner-mining-drill", count = 1}
		player.insert{name="stone-furnace", count = 1}
		player.insert{name="basic-bullet-magazine", count=10}
	else
		player.insert{name="basic-bullet-magazine", count=2}
	end
end

function newplayer(player)
	local force, surface = player.force, player.surface
	if forces[force.name] then
		--secondary player
		player.print{"placed",game.item_prototypes["raw-fish"].localised_name,game.item_prototypes[SUBMISSION_ENTITY].localised_name}
		givestartup(player)
		return
	end
	
	if not global.map_info then
		global.map_info= {}
		global.map_info.settings = surface.map_gen_settings
		if remote.interfaces.RSO then global.map_info.RSO = true end
	end
	
	local techs,recipes,items,needs = {},{},{},{}
	
	for _,entity in pairs(game.entity_prototypes) do
		if entity.autoplace_specification then addyield(items,entity) end
	end
	
	for _,recipe in pairs(force.recipes) do
		if recipe.enabled then recipes[recipe.name] = true end
	end

	for _,tech in pairs(force.technologies) do
		if tech.enabled then
			for _,effect in pairs(tech.effects) do
				if effect.type=="unlock-recipe" then recipes[effect.recipe]=true end
			end
		end
	end
	
	for recipe in pairs(recipes) do
		for _,product in pairs(force.recipes[recipe].products) do
			if product.type=="item" or product.type==0 then items[product.name]=true end
		end
	end
	
	items["raw-fish"]=nil
	
	local total = 0
	for item in pairs(items) do
		needs[item] = game.item_prototypes[item].stack_size*30
		total = total + 1
	end
	
	--needs={["iron-plate"]=2}

	player.insert{name=SUBMISSION_ENTITY,count=1}
	script.on_event(BUILDEVENTS,placedchest)

	player.print{"welcome",game.item_prototypes["raw-fish"].localised_name}
	player.print{"welcome2",game.item_prototypes[SUBMISSION_ENTITY].localised_name}

	--[[
	local place = surface.find_non_colliding_position("rocket-silo",force.get_spawn_position(surface),64,.5)
	if not place then
		place = surface.find_non_colliding_position(SUBMISSION_ENTITY,force.get_spawn_position(surface),64,.5)
	else
		place = surface.find_non_colliding_position("crude-oil",place,4.5,.5)
	end
	local chest = surface.create_entity{name=SUBMISSION_ENTITY,force=force,position=place}
	--]]
		
	forces[force.name]={needs=needs,
						--dest_chest = chest,
						suggest = next(needs),
						total = total,
						complete = 0,
						gameforce = force,
						}
	
	--game.write_file("30stack.txt",serpent.block(needs))
end

function placedchest(event)
	local entity = event.created_entity
	if entity.name~=SUBMISSION_ENTITY then return end
	
	local force = forces[entity.force.name]
	if force.dest_chest then return end
	
	force.dest_chest = entity
    entity.minable = false
    entity.destructible = false
	
	local pindx = event.player_index
	local flag = true
	broadcast(entity.force,{"placed",game.item_prototypes["raw-fish"].localised_name,game.item_prototypes[SUBMISSION_ENTITY].localised_name})
	if pindx then
		givestartup(game.get_player(pindx), true)
	else
		for _,player in pairs(entity.force.players) do
			givestartup(player,flag)
			flag = false
		end
		flag = true
	end
	
	for _,data in pairs(forces) do
		flag = data.dest_chest and flag
	end
	
	if flag then script.on_event(BUILDEVENTS, nil) end
end

script.on_event(defines.events.on_player_created, function(event)
	local player = game.get_player(event.player_index)
	local challenge = player.gui.top.add{type="frame", name="challenge", direction="vertical", caption = {"boxtitle"}}
	local box  = player.gui.top.challenge.add{type="table",name="box",direction="vertical",colspan=1}
	box.add{type="label", name="timer"}
	box.add{type="label",name="complete"}
	box.add{type="label", name="suggestion"}
  
	newplayer(player)
	update_clock(player.force.name)
	update_complete(player.force.name)
	update_suggestion(player.force.name)
end)

function update_clock()
	local ticks = 1 + TIME_LIMIT - game.tick
	local hours = floor(ticks/216000)
	ticks = ticks - hours*216000
	local minutes = floor(ticks/3600)
	ticks = ticks - minutes*(3600)
	local seconds = floor(ticks/60)
	
	local remain = format("%i:%02i:%02i",hours,minutes,seconds)
	
	for _,player in pairs(game.players) do
		if player.connected then
			player.gui.top.challenge.box.timer.caption = {"boxtimer",remain}
		end
	end
end

function update_suggestion(fname)
	local data = forces[fname]
	local talky = {"suggestion",game.item_prototypes[data.suggest].localised_name,data.needs[data.suggest]}
	if not data.suggest then talky = "" end
	for _,player in pairs(data.gameforce.players) do
		if player.connected then
			player.gui.top.challenge.box.suggestion.caption = talky
		end
	end
end

function update_complete(fname)
	local data = forces[fname]
	for _,player in pairs(data.gameforce.players) do
		if player.connected then
			player.gui.top.challenge.box.complete.caption = {"boxcomplete",data.complete,data.total}
		end
	end

end

function show_map_settings()
	local info, mapgui = global.map_info
	for _,player in pairs(game.players) do
		if info.RSO then
			mapgui = player.gui.left.add{type="frame",name="mapinfo",direction="vertical",caption={"challengemapsettings_withrso",info.settings.seed}}
		else
			mapgui = player.gui.left.add{type="frame",name="mapinfo",direction="vertical",caption={"challengemapsettings",info.settings.seed}}
		end
		mapgui = mapgui.add{type="table",name="table",colspan=1}
		
		mapgui.add{type="label",name="seg",caption={"basiclabel",{"gui-map-generator.terrain-segmentation"},{"frequency."..info.settings.terrain_segmentation}}}
		mapgui.add{type="label",name="h2o",caption={"basiclabel",{"gui-map-generator.water"},{"size."..info.settings.water}}}
		
		--segmentation
		--water
		--()
		--startingarea
		--size?
		
		for s,v in pairs(info.settings.autoplace_controls) do
			mapgui.add{type="label",name=s,caption={"autolabel",{"autoplace-control-names."..s},{"frequency."..v.frequency},{"size."..v.size},{"richness."..v.richness}}}
		end
		
		mapgui.add{type="label",name="start",caption={"basiclabel",{"gui-map-generator.starting-area"},{"size."..info.settings.starting_area}}}
		if info.settings.width+info.settings.height > 0 then
			mapgui.add{type="label",name="dim",caption={"basiclabel",{"gui-map-generator.size"},info.settings.width.." x "..info.settings.height}}
		end
		
		--mapgui.add{type="label",name="dump",caption=serpent.block(info.settings)}
		
		if info.settings.peaceful_mode then mapgui.add{type="label",name="peace",caption = {"gui-map-generator.peaceful-mode"}} end
	end
end

local tick, second, longtick
script.on_event(defines.events.on_tick, function(event)
	if tick == nil then
		tick = game.tick
		second = floor(tick/3)%20
		longtick = floor(tick/60)%5
		tick = tick % 3
	else
		if tick>0 then tick = tick - 1; return end
		if game.tick > TIME_LIMIT then
			for _,force in pairs(game.forces) do
				broadcast(force,{"lose"})
			end
			show_map_settings()
			game.set_game_state{game_finished=true,player_won=false}		
		end
		tick = 2
		if second>0 then second = second - 1 else
			update_clock()
			second = 19
			if longtick > 0 then longtick = longtick - 1 else
				longtick = 4
			end
		end
	end
	
	local inventory, remaining, localized
	
	for name,force in pairs(forces) do
		if force.dest_chest then
			inventory = force.dest_chest.get_inventory(CHESTINV)
			if not inventory.is_empty() then
				for item,count in pairs(inventory.get_contents()) do
					remaining = force.needs[item]
					localised = game.item_prototypes[item].localised_name
					if remaining then
						if count>=remaining then
							count=remaining
							remaining=0
						else
							remaining = remaining - count
						end
						inventory.remove{name=item,count=count}
						if remaining>0 then
							force.needs[item]=remaining
							broadcast(force.gameforce,{"accepted",localised,count,remaining})
						else
							force.complete = force.complete + 1
							broadcast(force.gameforce,{"completed",localised,count})
							update_complete(name)
							force.needs[item]=nil
							force.suggest = next(force.needs,item)
							if not force.suggest then force.suggest = next(force.needs) end

							if force.suggest then
								update_suggestion(name)
							else
								show_map_settings()
								broadcast(force.gameforce,{"win"})
								game.set_game_state{game_finished=true,player_won=true}
							end
						end
						update_suggestion(name)
					else
						if (longtick+second)==0 then
							broadcast(force.gameforce,{"unnecessary",localised})
						end
					end
				end
			end
		end
		if (longtick+second)==0 then
			force.suggest = next(force.needs,force.suggest)
			if not force.suggest then force.suggest = next(force.needs) end
			update_suggestion(name)
		end
	end
	
	
end)
